import { world, system, Direction } from "@minecraft/server";

const interactableBlockTypes = ["stairs", "slab", "log", "warped_stem", "crimson_stem", "wood", "hyphae", "creaking_heart", "bone_block", "bamboo_block"];
const cooldowns = new Set();
const activeSeats = new Map();

const initializeScript = () => {
  console.warn("§g§Take a Seat§r§a loaded§r - Author:§c xAssassin");

  system.runTimeout(() => {
    const dimensions = ["overworld", "nether", "the_end"];
    dimensions.forEach(dimensionName => {
      const dimension = world.getDimension(dimensionName);
      dimension.runCommand("kill @e[type=xassassin:sit]");
    });
  }, 1);
};

system.run(initializeScript);

world.beforeEvents.playerInteractWithBlock.subscribe((eventData) => {
  const player = eventData.player;
  if (player.isSneaking) return;
  const item = player.getComponent("minecraft:inventory")
                    .container.getItem(player.selectedSlotIndex);

  const block = eventData.block;
  const blockFace = eventData.blockFace;
  const blockLocation = block.location;
  const dimension = world.getDimension(player.dimension.id); 
  const playerId = player.id;

  const blockStates = block.permutation.getAllStates();
  const isSlab = block.typeId.includes("slab");
  const verticalHalf = blockStates["minecraft:vertical_half"];
  const isStairs = block.typeId.includes("stairs");
  const upsideDownBit = blockStates["upside_down_bit"];
  const weirdoDirection = isStairs ? blockStates["weirdo_direction"] : null;
  const isLog = block.typeId.includes("log") || block.typeId.includes("crimson_stem") || block.typeId.includes("wood") ||
block.typeId.includes("bamboo_block") ||
block.typeId.includes("hyphae") ||
block.typeId.includes("bone_block") || block.typeId.includes("creaking_heart") || block.typeId.includes("warped_stem");
  const seatEntities = dimension.getEntities().filter(entity => {
    if (entity.typeId !== "xassassin:sit") return false;
    const entityLocation = entity.location;
    return (
      Math.floor(entityLocation.x) === Math.floor(blockLocation.x) &&
      Math.floor(entityLocation.y) === Math.floor(blockLocation.y) &&
      Math.floor(entityLocation.z) === Math.floor(blockLocation.z)
    );
  });

  seatEntities.forEach(seatEntity => {
    const nearbyPlayers = world.getPlayers().filter(p => {
      const distance = Math.sqrt(
        Math.pow(p.location.x - seatEntity.location.x, 2) +
        Math.pow(p.location.y - seatEntity.location.y, 2) +
        Math.pow(p.location.z - seatEntity.location.z, 2)
      );
      return distance <= 0.25;
    });

    if (nearbyPlayers.length === 0) {
      system.runTimeout(() => {
        seatEntity.remove();
      }, 1);
    }
  });

  if (block.typeId.includes("door")) return;

  if (!interactableBlockTypes.some(type => block.typeId.includes(type))) return;

  if ((isSlab && verticalHalf === "top") || (isStairs && upsideDownBit === true)) return;

  const invalidItemNames = [
  "bricks", "slab", "chest", "table", "log", "block", "snow", "head", "skull", 
  "carpet", "door", "stair", "chair", "anvil", "brew", "wall", "sign", "cracked", 
  "slate", "gate", "egg", "bell", "wool", "glazed", "banner", "ore", "cobblestone", 
  "stone", "box", "cake", "vine", "skulk", "clay", "cobbled", "copper", "heart", 
  "crafter", "dispenser", "dropper", "rail", "lamp", "lectern", "path", "obsidian", 
  "frame", "eye", "splash", "charge", "steel", "beacon", "furnace", "smoker", "moss", 
  "pot", "candle", "boat", "spawn", "vault", "button", "torch", "hook", "tnt", 
  "redstone", "planks", "fence", "plate", "stripped", "camp", "lava", "water", "ice", 
  "magma", "coral", "bucket", "wrench", "debug", "light", "sand", "glass", "stem", 
  "end", "piston", "wax", "amethyst", "compost", "dirt", "mud", "granite", "lever", 
  "hyphae", "wood", "iron_bars", "leaves", "flower", "resin", "hive", "nest", 
  "backpack", "lantern", "stand", "dummy", "target", "sea_pickle", "spounge", 
  "andesite", "drip", "rod", "roots", "haybale", "pumpkin", "jack"
];

const distanceXZ = Math.sqrt(Math.pow(player.location.x - blockLocation.x, 2) + Math.pow(player.location.z - blockLocation.z, 2));
const hasInvalidItemName = invalidItemNames.some(name => item?.typeId.includes(name)) && 
                          !(item?.typeId.includes("wood") && item?.hasTag("minecraft:is_tool"));

if (!hasInvalidItemName && distanceXZ > 3) {
  system.runTimeout(() => {
    player.onScreenDisplay.setActionBar("§7Block is too far to sit on");
  }, 1);
  return;
}

  if (hasInvalidItemName && !item?.hasTag("minecraft:is_tool")) return;

  if (item?.hasTag("xassassin:wrench")) return;

  if (isSlab && block.typeId.includes("double_slab")) return;

  if (block.typeId.includes("copper") && item?.hasTag("minecraft:is_axe")) return;

  if (isLog && item?.hasTag("minecraft:is_axe")) return;

  if (blockFace === Direction.Down) return;

  const blockAbove1 = dimension.getBlock({ x: blockLocation.x, y: blockLocation.y + 1, z: blockLocation.z });
  const blockAbove2 = isLog ? dimension.getBlock({ x: blockLocation.x, y: blockLocation.y + 2, z: blockLocation.z }) : null;

  const isBreathableBlock = (typeId) => (
    typeId === "minecraft:air" ||
    typeId.includes("sign") ||
    typeId === "minecraft:frame" ||
    typeId === "minecraft:glow_frame" ||
    typeId === "minecraft:painting" ||
    typeId === "minecraft:banner" ||
    typeId.includes("gate") ||
    typeId.includes("door") ||
    typeId.includes("button") ||
    typeId.includes("torch") ||
    typeId.includes("lever") ||
    typeId.includes("rod") ||
    typeId.includes("web") ||
    typeId.includes("hanging") ||
    typeId.includes("berry") ||
    typeId.includes("carpet") ||
    typeId.includes("chain") 
  );

  if (!isBreathableBlock(blockAbove1.typeId) || (isLog && !isBreathableBlock(blockAbove2.typeId))) return;

  const entities = dimension.getEntities();
  const isBlockOccupied = entities.some(entity => {
    if (entity.typeId !== "xassassin:sit") return false;
    const entityLocation = entity.location;
    return (
      Math.floor(entityLocation.x) === Math.floor(blockLocation.x) &&
      Math.floor(entityLocation.y) === Math.floor(blockLocation.y) &&
      Math.floor(entityLocation.z) === Math.floor(blockLocation.z)
    );
  });

  if (isBlockOccupied) return;
  const playerY = Math.floor(player.location.y);
  if (blockLocation.y - playerY >= 2 || playerY - blockLocation.y >= 3) {
 system.runTimeout(() => {
    player.onScreenDisplay.setActionBar("§7Block is too far to sit on");
  }, 1);
   return;
}

  if (!player.isOnGround) return;

  const nearbySeatEntities = entities.filter(entity => {
    if (entity.typeId !== "xassassin:sit") return false;
    const distance = Math.sqrt(
      Math.pow(entity.location.x - player.location.x, 2) +
      Math.pow(entity.location.y - player.location.y, 2) +
      Math.pow(entity.location.z - player.location.z, 2)
    );
    return distance <= 0.5;
  });

  if (nearbySeatEntities.length > 0) return;

  if (cooldowns.has(playerId)) return; 
  cooldowns.add(playerId); 

  system.runTimeout(() => {
    let seatRotation = player.getRotation();

    if (isStairs) {
      switch (weirdoDirection) {
        case 0:
          seatRotation = { x: 0, y: 90 };
          break;
        case 1:
          seatRotation = { x: 0, y: 270 };
          break;
        case 2:
          seatRotation = { x: 0, y: 180 };
          break;
        case 3:
          seatRotation = { x: 0, y: 0 };
          break;
      }
    }

    const seatSpawnLocation = {
      x: blockLocation.x + 0.5,
      y: isLog ? blockLocation.y + 0.5 : blockLocation.y,
      z: blockLocation.z + 0.5,
    };

    const seat = dimension.spawnEntity("xassassin:sit", seatSpawnLocation);
    seat.setRotation(seatRotation);
    seat.getComponent("rideable").addRider(player);
    activeSeats.set(seat.id, blockLocation);

    const checkInterval = system.runInterval(() => {
      const seatEntity = world.getEntity(seat.id);
      if (!seatEntity) {
        system.clearRun(checkInterval);
        return;
      }

      const originalBlockLocation = activeSeats.get(seat.id);
      if (!originalBlockLocation) return;

      const currentBlock = dimension.getBlock(originalBlockLocation);
      const isBlockRemoved = currentBlock.typeId === "minecraft:air" ||
                             currentBlock.typeId === "minecraft:water" ||
                             currentBlock.typeId === "minecraft:sticky_piston_arm_collision" ||
                             currentBlock.typeId === "minecraft:piston_arm_collision";

      const nearbyPlayers = world.getPlayers().filter(p => {
        const distance = Math.sqrt(
          Math.pow(p.location.x - seatEntity.location.x, 2) +
          Math.pow(p.location.y - seatEntity.location.y, 2) +
          Math.pow(p.location.z - seatEntity.location.z, 2)
        );
        return distance <= 0.5;
      });

      if (isBlockRemoved || nearbyPlayers.length === 0) {
        seatEntity.remove();
        activeSeats.delete(seat.id);
        system.clearRun(checkInterval);
      }
    }, 10);

    system.runTimeout(() => {
      cooldowns.delete(playerId); // Remove player from cooldown after 6 ticks
    }, 6);
}, 5);
});

system.runInterval(() => {
  cooldowns.forEach((playerId) => {
    const player = world.getPlayer(playerId);
    if (!player) {
      cooldowns.delete(playerId);
    }
  });
}, 100);
